import * as mc from "@minecraft/server";
import * as ui from "@minecraft/server-ui";
import { ItemStack, system } from "@minecraft/server";

export function addScore(player, objective, value) { player.runCommandAsync("scoreboard players add @s " + objective + " " + value); }

export function getScore(objective, player) { try { return mc.world.scoreboard.getObjective(objective).getScore(player.scoreboardIdentity); } catch (error) { return 0; } }

export function removeScore(player, objective, value) { player.runCommandAsync("scoreboard players remove @s " + objective + " " + value); }

export function setScore(player, objective, value) { player.runCommandAsync("scoreboard players set @s " + objective + " " + value); }

export const overworld = mc.world.getDimension("overworld"), nether = mc.world.getDimension("nether"), theEnd = mc.world.getDimension("the end");

export function number(value) { const ranges = [ { divider: 1e9, suffix: "b" }, { divider: 1e6, suffix: "m" }, { divider: 1e3, suffix: "k" }, ]; for (const range of ranges) if (value >= range.divider) return ((value / range.divider).toFixed(1) + range.suffix).toString(); return value; }

export function tick1(callback) { mc.system.runInterval((data) => { for (const player of mc.world.getPlayers()) { callback(player); } }, 1); }

export function tick5(callback) { mc.system.runInterval((data) => { for (const player of mc.world.getPlayers()) { callback(player); } }, 5); }

export function tick10(callback) { mc.system.runInterval((data) => { for (const player of mc.world.getPlayers()) { callback(player); } }, 10); }

export function random(min, max) { return Math.floor(Math.random() * (max - min + 1)) + min; }

export function ItemDurability(player,equipmentSlot){
    const equippableComponent = player.getComponent("equippable");
    const mainhandItem = equippableComponent.getEquipment(equipmentSlot);
	if(mainhandItem==undefined) return;
    const itemDurability = mainhandItem.getComponent("durability");
	if(itemDurability==undefined) return;
	const maxDurability = itemDurability.maxDurability;
	const damagedDurability = itemDurability.damage;
	const remainingDurability = maxDurability - damagedDurability;
	if(remainingDurability > 0){
		const updateItemStack = new ItemStack(mainhandItem.typeId, 1);
		const updateItemDurabilityComp = updateItemStack.getComponent("durability");
		updateItemDurabilityComp.damage = damagedDurability + 1;
		equippableComponent.setEquipment(equipmentSlot,updateItemStack);
	}
	else {
		equippableComponent.setEquipment(equipmentSlot,undefined);
	}
}

export function getItemAmount(player, itemType) {
  let amount = 0
  const inv = player.getComponent("inventory").container
  for(let i = 0; i < inv.size; i++) {
    const item = inv.getItem(i)
    if(item?.typeId === itemType) amount += item.amount
  }
  return amount
}

export function hasMainhandItemTag(entity,tag){
    const inventory = entity.getComponent("minecraft:equippable");
    if(inventory.getEquipment("Mainhand") != null){
        const mainhandItemTag = inventory.getEquipment("Mainhand").hasTag(tag);
        return mainhandItemTag;
    }
}

export function hasEquipmentItem(player,equipmentSlot,itemName){
    const equippableComponent = player.getComponent("minecraft:equippable");
    if(equippableComponent.getEquipment(equipmentSlot)!=undefined){
        return equippableComponent.getEquipment(equipmentSlot).typeId == itemName;
    }else{
        return false;
    }
}

export function shootPower(player, powerId, offset, eventId, name, speed) {
  if (offset != null) {
    const location = offsetPower(player.getHeadLocation(),player.getRotation(),offset);
    common(location);
  } else {
    const location = player.getHeadLocation();
    common(location);
  }
  function common(location) {
    const power = player.dimension.spawnEntity(powerId,location);
    if(name != null){
      power.nameTag = `${name}`;
    }
    if(eventId != null){
      power.triggerEvent(eventId);
    }
    const projectileComp = power.getComponent("projectile");
    projectileComp.owner = player;
    if (speed == null) {
      projectileComp.shoot(player.getViewDirection());
    } else projectileComp.shoot({x:speed,y:speed,z:speed});
  }
}

export function summonPower2(player, powerId, eventId, name, offsetX, offsetY, offsetZ) {
  const view = player.getViewDirection();
  player.runCommandAsync(`summon ` + powerId + ` ` + Math.floor(player.location.x + view.x * offsetZ) + ` ` + Math.floor(player.location.y) + ` ` + Math.floor(player.location.z + view.z * offsetZ) + ` ~` +
  //Math.floor(player.getRotation().y) +
  ` ` + Math.floor(player.getRotation().y - player.getRotation().y) + ` ` + eventId + ` "` + name + `"`);
}

export function summonPower(player, powerId, offsetX, offsetY, offsetZ, eventId, name) {
  const rotationY = (player.getRotation().y + 90) * (Math.PI / 180);
  const
    viewX = Math.cos(rotationY),
    viewZ = Math.sin(rotationY),
    x = Math.floor(player.location.x + viewX * offsetZ),
    y = Math.floor(player.location.y + offsetY),
    z = Math.floor(player.location.z + viewZ * offsetZ);
  player.runCommandAsync(`summon ${powerId} ${x} ${y} ${z} ~ ${Math.floor(player.getRotation().y - player.getRotation().y)} ${eventId} "${name != null ? name : ""}"`);
}

export function offsetPower(location, rotation, offset) {
    function round100000(number) {
        return Math.round(100000*number)/100000;
    }
    const x_entityRotationY = (rotation.y + 90) * (Math.PI/180);
    const x_positionX = -round100000(Math.sin(x_entityRotationY));
    const x_positionZ = round100000(Math.cos(x_entityRotationY));
    
    const z_entityRotationY = rotation.y * (Math.PI/180);
    const z_entityRotationX = rotation.x * (Math.PI/180);
    const z_positionX = -round100000(Math.sin(z_entityRotationY)*Math.cos(z_entityRotationX));
    const z_positionZ = round100000(Math.cos(z_entityRotationY)*Math.cos(z_entityRotationX));
    const z_positionY = -round100000(Math.sin(z_entityRotationX));
    
    const y_entityRotationY = rotation.y * (Math.PI/180);
    const y_entityRotationX = rotation.x * (Math.PI/180);
    const y_positionX = -round100000(Math.sin(y_entityRotationY)*Math.sin(y_entityRotationX));
    const y_positionZ = round100000(Math.cos(y_entityRotationY)*Math.sin(y_entityRotationX));
    const y_positionY = round100000(Math.cos(y_entityRotationX));

    return {
        x:location.x + x_positionX*offset[0] + y_positionX*offset[1] + z_positionX*offset[2],
        y:location.y + z_positionY*offset[2] + y_positionY*offset[1],
        z:location.z + x_positionZ*offset[0] + y_positionZ*offset[1] + z_positionZ*offset[2]
    }
}
































       
             
                   
                                   
                   
                              
                   
                              
                   
                              
                   
                              
                   
                              
                   
                              
                   
                              
                   
                             
                              
                   
                              
                   
                              
                   
                              
                   
                                               
                   
                              
                   
                              
                   
                             
                              
                   
                              
                   
                              
                   
                              
                   
                          
                              
                          
                                                    
                             
                              
                   
                              
                   
                              
                   
                              
                   
                                               
                   
                              
                   
                              
                   
                             
                              
                   
                              
                   
                              
                   
                              
                   
                          
                              
                          
                                                                                                                                                    
                                                                                                                                                                                                      
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           