import * as mc from "@minecraft/server";
import * as q from "./list.js";
import { addScore, getScore, random, setScore } from "./all.js";

mc.system.runInterval(() => {
  for (const player of mc.world.getPlayers({ tags: ["start_mission"] })) {
    const mission = q.Missions.list[getScore("missions", player)];
    player.removeTag("start_mission");
    if (!player.hasTag("in_mission")) {
      player.runCommandAsync(`tellraw @s{"rawtext":[{"translate":"menu.missions.open"}]}`);
      player.addTag("in_mission");
    }
    inMission(player, mission);
  }
}, 20);

function inMission(player, mission) {
  if (player.hasTag("finish_mission")) finishMission(player, mission);
  if (player.getComponent("minecraft:health")?.currentValue <= 0) {
    missionFailed(player, mission);
  }
  if (mission.enemies != null) {
    let enemiesCount = {};
    for (const enemy of mission.enemies) {
      enemiesCount[enemy] = (enemiesCount[enemy] || 0) + 1;
    }
    mc.world.afterEvents.entityHurt.subscribe(data => {
      try {
        if (data.damageSource.damagingEntity === player) {
          const enemyType = data.hurtEntity.typeId;
          if (enemiesCount[enemyType] && data.hurtEntity.getComponent("minecraft:health")?.currentValue <= 0) {
            enemiesCount[enemyType]--;
            if (player.hasTag("in_mission") && Object.values(enemiesCount).every(count => count <= 0)) {
              finishMission(player, mission);
            }
          }
        }
      }
      catch (error) { }
    });
  }
  if (mission.items != null) {
    const objectiveItems = {};
    mission.items.forEach(item => {
      if (objectiveItems[item]) {
        objectiveItems[item]++;
      } else {
        objectiveItems[item] = 1;
      }
    });

    const time = mc.system.runInterval(() => {
      try {
        let allItemsCollected = true;

        for (const itemType in objectiveItems) {
          const requiredAmount = objectiveItems[itemType];
          const playerAmount = getItemAmount(player, itemType);

          if (playerAmount < requiredAmount) {
            allItemsCollected = false;
            break;
          }
        }

        if (player.hasTag("in_mission") && allItemsCollected) {
          mc.system.clearRun(time);
          finishMission(player, mission);
        }
      } catch (error) {
        console.error(error);
      }
    }, 20);
  }
}

function finishMission(player, mission) {
  player.runCommandAsync("tag @s remove finish_mission");
  player.runCommandAsync("tag @s remove in_mission");
  if (mission.money || mission.tp) {
    player.runCommandAsync(`tellraw @s {"rawtext":[{"text": "§e<"},{"translate": "mission.complete"},{"text": ">\n§f"},{"translate": "mission.earned"},{"text": " §a${mission.money} Ryu${mission.money > 1 ? "s" : ""}\n§f"},{"translate": "mission.earned"},{"text": " §e${mission.tp} Tp.§f"}]}`);
  } else player.runCommandAsync(`tellraw @s {"rawtext":[{"text": "§e<"},{"translate": "mission.complete"},{"text": ">§f"}]}`);
  
  if (mission.items) {
    const objectiveItems = {};
    mission.items.forEach(item => {
      if (objectiveItems[item]) {
        objectiveItems[item]++;
      } else {
        objectiveItems[item] = 1;
      }
    });
    for (const itemType in objectiveItems) {
      const requiredAmount = objectiveItems[itemType];
      player.runCommandAsync(`clear @s ${itemType} 0 ${requiredAmount}`);
    }
  }
  if (mission.give) {
    const giveItems = {};
    mission.give.forEach(gitem => {
      if (giveItems[gitem]) {
        giveItems[gitem]++;
      } else {
        giveItems[gitem] = 1;
      }
    });
    for (const itemType in giveItems) {
      const requiredAmount = giveItems[itemType];
      player.runCommandAsync(`give @s ${itemType} ${requiredAmount}`);
    }
  }
  if (mission.givetag) {
    const giveItems = {};
    mission.givetag.forEach(gitem => {
      if (giveItems[gitem]) {
        giveItems[gitem]++;
      } else {
        giveItems[gitem] = 1;
      }
    });
    for (const itemType in giveItems) {
      player.runCommandAsync(`tag @s add ${itemType}`);
    }
  }
  setScore(player, "missions", 0);
  if (mission.money) addScore(player, "money", mission.money);
  if (mission.tp) addScore(player, "tp", mission.tp);
}

function missionFailed(player, mission) {
  player.runCommandAsync("tag @s remove finish_mission");
  player.runCommandAsync("tag @s remove in_mission");
  player.runCommandAsync(`tellraw @s {"rawtext":[{"text": "§e<"},{"translate": "mission.failed"},{"text": ">\n§f"}]}`);
  setScore(player, "missions", 0);
}

function getItemAmount(player, itemType) {
  let amount = 0
  const inv = player.getComponent("inventory").container
  for(let i = 0; i < inv.size; i++) {
    const item = inv.getItem(i)
    if(item?.typeId === itemType) amount += item.amount
  }
  return amount
}

                                    
                                    
                                    
                                    
                                    
                                    
                                    
                                    
                                    
                                    
                                    
                                    
                                    
                                    
                                                       
                                    
                                    
                                    
                                    
                                    
                                    
                                                      
                                                
                                                
                                                                                   
                                                      
                                                
                                                
                                                                                            
                                    
                                    
                                    
                                    
                                                       
                                    
                                    
                                    
                                    
                                    
                                    
                                                      
                                                
                                                
                                                                                   
                                                      
                                                
                                                
                                                                                                                                                                                                 
                                                      
                                                
                                                
                                                                                                                                                                     
                                                      
                                                
                                                
                                                                                                                                                                                                 
                                                      
                                                
                                                
                                                                                                                       